/* 
 * keyshades
 * a tool for disabling the treo keyboard backlight
 * copyright (c) 2005, daniel grobe sachs <dgsachs@nekito.net>
 *
 * This is free software. See included LICENSE file for details.
 */

#include <PalmOS.h>
#include <palmOne_68K.h>
#include <68K/System/HsExt.h>
#include <Common/System/HsExtCommon.h>

#define FTR_HANDLE	0
#define FTR_ONEHAND	1 	// Are we in one-hand mode?

static void RegisterForNotifications();
static void DeregisterNotifications();

// Endian swapper
#define EndianSwap16(n) (((((UInt16) n) << 8) & 0xFF00) | \
                         ((((UInt16) n) >> 8) & 0x00FF))


// Notification handler
static Err HandleNotification(SysNotifyParamType *np)
{
	UInt32 val;
	struct EventType newevent;
	EventType *eventP = (EventType*)np->notifyDetailsP;
	FormType *formP;

	// Check if we're deleting the application
	if( np->notifyType == sysNotifyDeleteProtectedEvent )
	{
		if( ((SysNotifyDBInfoType *)np->notifyDetailsP)->creator == CREATOR )
		{
			DeregisterNotifications();
			np->handled = true;
		}

		return 0;
	}

	// Is this a queue event?
	if( np->notifyType != sysNotifyEventDequeuedEvent )
		return 0;

	// Is this a keyboard event?
	if( (eventP->eType != EndianSwap16(keyDownEvent)) )
		return 0;

	// Is there a form?
	formP = FrmGetActiveForm();
	if( formP == NULL )
		return 0;	// no - keep FieldPlus in the dark

	// Is form numbered 0?
	if( FrmGetActiveFormID() == 0 )
		return 0;	// no - keep FieldPlus in the dark

        // Is this a key we are interested in?
        switch( EndianSwap16(eventP->data.keyDown.keyCode) )
        {
		// Activate one-hand mode
		case keyLeftAlt: 	// option
		case keyLeftShift:
		case keyRightShift:
			FtrSet(CREATOR, FTR_ONEHAND, 1);
			// (falls through to send)
			
		// Always send the following keycodes. 
		case keyRockerUp:
		case keyRockerDown:
		case keyRockerLeft:
		case keyRockerRight:
		case keyRockerCenter:
		case keyMenu:
		case keyLaunch:
		case keyFind:
		case keyPageUp:
		case keyPageDown:
			np->notifyType = 'FpNh';
			SysNotifyBroadcast(np);
			np->notifyType = sysNotifyEventDequeuedEvent;
			break;

		default:
			// Only send if we're in one-hand mode
			FtrGet(CREATOR, FTR_ONEHAND, &val);
			if( !val )
				break;

			FtrSet(CREATOR, FTR_ONEHAND, 0);

			np->notifyType = 'FpNh';
			SysNotifyBroadcast(np);
			np->notifyType = sysNotifyEventDequeuedEvent;
			break;
	}

	return 0;
}


#define STARTUP_CODE_SIZE	152		// size of prc-tools crt0.c
#define FTR_PTR			999

static void RegisterForNotifications()
{
	MemHandle hndl;
	void *memptr;
	void *ftrptr;
	void *hn_ptr;

	UInt16 cardNo;
	LocalID dbID;

	if( 0 != SysCurAppDatabase(&cardNo, &dbID) )
		return;

	hndl = DmGet1Resource('code',1);
	memptr = MemHandleLock(hndl);

	FtrPtrNew(CREATOR, FTR_PTR, MemHandleSize(hndl),&ftrptr);
	DmWrite(ftrptr, 0, memptr, MemHandleSize(hndl));

	MemHandleUnlock(hndl);
	DmDatabaseProtect(cardNo, dbID, true);

	hn_ptr = ((void *)ftrptr)+STARTUP_CODE_SIZE;

	SysNotifyRegister(cardNo, dbID, sysNotifyEventDequeuedEvent, hn_ptr, 
			sysNotifyNormalPriority, NULL);

 	SysNotifyRegister(cardNo, dbID, sysNotifyDeleteProtectedEvent, NULL,
			sysNotifyNormalPriority, NULL);
}

static void DeregisterNotifications()
{	
	Int32 val;
	void *ftrptr;

	UInt16 cardNo;
	LocalID dbID;

	val = 0; FtrGet(CREATOR, FTR_PTR, &val); ftrptr = (void *)val;

	// Check if we're registered
	if( ftrptr == NULL )
		return; 

	if( 0 != SysCurAppDatabase(&cardNo, &dbID) )
		return;

	SysNotifyUnregister(cardNo, dbID, sysNotifyEventDequeuedEvent,
			sysNotifyNormalPriority);

 	SysNotifyUnregister(cardNo, dbID, sysNotifyDeleteProtectedEvent,
			sysNotifyNormalPriority);

	// release feature memory
	FtrPtrFree(CREATOR, FTR_PTR);
	DmDatabaseProtect(cardNo, dbID, false);
}


//// PilotMain /////////////////////////////////////////////////////////
// It all starts here...

UInt32 PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags) 
{
	Err err = 0;
	EventType event;

	switch( launchCode ) 
	{
		case sysAppLaunchCmdSystemReset :
			FtrSet(CREATOR, FTR_ONEHAND, 0);
			RegisterForNotifications();
			break;

		case sysAppLaunchCmdNotify : 
			HandleNotification((SysNotifyParamType *)cmdPBP);
			break;

		case sysAppLaunchCmdNormalLaunch :
			// Well, uh, we don't have a UI, so we just quit
			MemSet( &event, sizeof(event), 0);

			event.eType = appStopEvent;
			EvtAddEventToQueue(&event);
			break;

		default:
			break;
	}


	return err;
}

